﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using PlayWav;



namespace Keyboard1
{
    public partial class Form1 : Form
    {
        const int COUNT = 40;
        const int SMALLFONT = 14;
        const int LARGEFONT = 32;
        const string FONT = "Arial";
        const string INSTRUCTIONS = "Select keyboard type, size, and input style. Have the subject press Start when ready to start the trial, and Stop when done.";
        private Point LARGE = new Point (800,600);
        private Point SMALL = new Point (500, 350);
  
        //Data
      
        private TextWriter tw;
        private string dataFile = "anonymous.txt";
        private int data_trial = 0;
        private string data_keyboardType = "Qwerty";
        private string data_size = "Large";
        private DateTime data_startTime;
        private string data_category = "Keyboard typing";


        //Keyboards
        private string[] qwertyArr = { "1", "2", "3", "4", "5", "6", "7", "8", "9", "0", 
                               "q", "w", "e", "r", "t", "y", "u", "i", "o", "p", 
                               "a", "s", "d", "f", "g", "h", "j", "k", "l", ";", 
                               "z", "x", "c", "v", "b", "n", "m", ",", ".", "/"};
        private string[] alphaArr =  { "1", "2", "3", "4", "5", "6", "7", "8", "9", "0", 
                               "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", 
                               "k", "l", "m", "n", "o", "p", "q", "r", "s", ";",
                               "t", "u", "v", "w", "x", "y", "z", ",", ".", "/"};

        private string[] randomArr =  { "1", "2", "3", "4", "5", "6", "7", "8", "9", "0", 
                               "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", 
                               "k", "l", "m", "n", "o", "p", "q", "r", "s", ";",
                               "t", "u", "v", "w", "x", "y", "z", ",", ".", "/"};


        //States
        private bool boolStarted = false;
        private bool boolShifted = false;
        private string [] selectedArray;

        //Interface stuff
        private Button[] btnArr = new Button[COUNT];

        #region Form load / unload
        public Form1()
        {
            InitializeComponent();

            //Add buttons to the keyboard and reset the keyboard to match the radio button configuration
            ButtonsCreate(COUNT);
            GenerateRandomKeyboard();
            resetKeyboard();
         
            // Put some initial instructions in the text field
            textBox1.Text = INSTRUCTIONS;
            textBox1.Focus();
            textBox1.Enabled = false;

            // Specify and open the data file, plus write initial information
            saveFileDialog1.ShowDialog();
            tw = new StreamWriter(this.dataFile);
            tw.WriteLine("Subject:  " + this.dataFile);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Make sure all logged data is written, then close the data file
            tw.Flush();
            tw.Close();
        }
        #endregion

        #region Generate a keyboard by shuffling the existing keys
        private void GenerateRandomKeyboard()
        {
            Random rand = new Random();
            int seed = rand.Next(0, 10000);
            int newKey = 0;
            string letter = " ";
            //Only shuffle the letter keys. The keys we want to shuffle are 10-28, 30-36
            for (int i = 10; i <= 36; i++)
            {
                if (i == 29 || i == 30) continue; //Skip 29 and 30
                // Get a new valid value for a key
                newKey = GenerateAKeyPosition(seed + i);
                letter = this.randomArr[i ];
                this.randomArr[i] = this.randomArr[newKey];
                this.randomArr[newKey] = letter;
            }
        }

        private int GenerateAKeyPosition(int seed)
        {
            Random rand = new Random(seed);
            int newKey = 0;

            // Get a new valid value for a key
            newKey = rand.Next(10, 36);
            while (newKey == 29 || newKey == 30) newKey = rand.Next(10, 36);
            return newKey;
        }
        #endregion

        #region Button setup

        // Create the buttons representing the keys in the table
        private void ButtonsCreate(int count)
        {
            for (int i = 0; i < count; i++)
            {
                btnArr[i] = new Button();
                btnArr[i].Dock = DockStyle.Fill;
                this.btnArr[i].Click += new EventHandler(btn_Click);
                this.tblpanelKeyboard.Controls.Add(btnArr[i]);
            }
        }

        // Set the fonts in the keyboard
        private void ButtonsSetFont(int count, string font, float size)
        {
            for (int i = 0; i < count; i++)
            {
                btnArr[i].Font = new Font(font, size);
            }
            btnSpace.Font = new Font(font, size);
            btnShift.Font = new Font(font, size - 6);
            btnBackSpace.Font = new Font(font, size - 6);
        }

        // Set the text in the keyboard to match the array defining the keyboard layout
        private void ButtonsSetText(int count, string[] kbArr)
        { 
            for (int i = 0; i < count; i++)
            {
                if (boolShifted)
                {
                    btnArr[i].Text = kbArr[i].ToUpper();
                }
                else
                {
                    btnArr[i].Text = kbArr[i];
                }
                this.selectedArray = kbArr;
            }
        }
        #endregion

        #region Button handlers 

        // For the next three functions: Whenever a keyboard button is clicked 
        // (all letters/numbers, space bar, backspace) and its in mouse typing mode, 
        //  play a sound, do the right action in the text display area, and 
        // record the key in the log file
        void btn_Click(object sender, EventArgs e)
        {
            if (!boolStarted || data_category != "Mouse typing") return;
            PlayWav.PlayWav.Play("typing_click.wav");
            Button btn = (Button)sender;
            this.textBox1.Text += btn.Text;
            tw.Write(btn.Text);
            if (this.boolShifted)
            {
                this.boolShifted = false;
                ButtonsSetText(COUNT, this.selectedArray);
            }
        }

        private void btnSpace_Click(object sender, EventArgs e)
        {
                if (!boolStarted || data_category != "Mouse typing") return;
                PlayWav.PlayWav.Play("typing_click.wav");
                this.textBox1.Text += " ";
                tw.Write(" ");
           
                // If the shift key was pressed (i.e., we are displaying capital letters)
                // reset the keys to lower case
                if (this.boolShifted)
                {
                    this.boolShifted = false;
                    ButtonsSetText(COUNT, this.selectedArray);
                }
        }

        private void btnBackSpace_Click(object sender, EventArgs e)
        {
            if (!boolStarted) return;
            string str = this.textBox1.Text;
            int new_length = str.Length - 1;

            //Click sound
            PlayWav.PlayWav.Play("typing_click.wav");

            //Trim a character
            if (new_length <= 0)
            {
                str = "";
            }
            else
            {
                str = str.Remove(new_length);
            }
            this.textBox1.Text = str;
            tw.Write("<BACKSPACE>");

            if (this.boolShifted)
            {
                this.boolShifted = false;
                ButtonsSetText(COUNT, this.selectedArray);
            }
        }

 
        // If the shift button is pressed, display the keyboard in capital letters
        // play a sound, and log it to the file.
        private void btnShift_Click(object sender, EventArgs e)
        {
            if (!boolStarted || data_category != "Mouse typing") return;
            this.boolShifted = true;
            ButtonsSetText(COUNT, this.selectedArray);

            PlayWav.PlayWav.Play("typing_click.wav");
            tw.Write("<Shift>");
        }

 
        #endregion

        #region Data printing

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            dataFile = saveFileDialog1.FileName;
        }


        private void TrialStart()
        {
            tw.WriteLine("");
            tw.WriteLine("============================================");
            tw.WriteLine("Trial:       " + this.data_trial.ToString());
            tw.WriteLine("Category:    " + this.data_category);
            tw.WriteLine("Keyboard:    " + this.data_keyboardType.ToString());
            tw.WriteLine("Size:        " + this.data_size.ToString ());
            tw.WriteLine("Start time:  " + DateTime.Now);
            this.data_startTime = DateTime.UtcNow;
        }


        private void TrialEnd ()
        {
            System.TimeSpan ElapsedTime = DateTime.UtcNow - this.data_startTime;

            double elapsed = ElapsedTime.Seconds  + (ElapsedTime.Minutes * 60);
            tw.WriteLine("");
            tw.WriteLine("Final text:  " + textBox1.Text);
            tw.WriteLine("Stop time:   " + DateTime.Now);
            tw.WriteLine("Elapsed time:   " + elapsed.ToString ());
            float lettersPerMinute = ((float) textBox1.Text.Length / (float) elapsed)  * 60;
            tw.WriteLine("Letters/minute: " + lettersPerMinute.ToString ());
            textBox1.Text = INSTRUCTIONS;
        }
        #endregion
        
        #region Keyboard redisplay
 
        private void resetKeyboard()
        {
            if (rbQwerty.Checked)
            {
                data_keyboardType = "Qwerty";
                ButtonsSetText(COUNT, qwertyArr);
            }
            else if (rbAlpha.Checked)
            {
                data_keyboardType = "Alpha";
                ButtonsSetText(COUNT, alphaArr);
            }
            else if (rbRandom.Checked)
            {
                data_keyboardType = "Random";
                ButtonsSetText(COUNT, randomArr);
            }
            if (rbLarge.Checked)
            {
                this.Width = LARGE.X;
                this.Height = LARGE.Y;
                ButtonsSetFont(COUNT, FONT, LARGEFONT);
                this.data_size = "Large";
            }
            else if (rbSmall.Checked)
            {
                this.Width = SMALL.X;
                this.Height = SMALL.Y;
                ButtonsSetFont(COUNT, FONT, SMALLFONT);
                this.data_size = "Small";
            }
        }
        #endregion

        #region Start/Stop a trial
        private void btnStart_Click(object sender, EventArgs e)
        {
            if (btnStart.Text == "Start")
            {
                this.boolStarted = true;
                TrialStart();
                textBox1.Text = "";
                btnStart.Text = "Stop";
                tw.Write("Typed text:  ");
                groupBox1.Visible = false;
                groupBox2.Visible = false;
                groupBox3.Visible = false;
                
                if (rbQwerty.Checked)
                    this.Text = "Qwerty";
                else if (rbAlpha.Checked)
                    this.Text = "Alphabetic";
                else if (rbRandom.Checked) 
                     this.Text =  "Random";
        
                this.Text += (rbLarge.Checked) ? "large, " : "small, ";
                this.Text += (rbKeyboard.Checked) ? "keyboard typing" : "mouse typing";
                textBox1.Enabled = true;
                textBox1.Focus();
                if (data_category != "Mouse typing")
                {
                    textBox1.Focus();
                }
            }
            else
            {
                this.boolStarted = false;
                textBox1.Enabled = false;
                groupBox1.Visible = true;
                groupBox2.Visible = true;
                groupBox3.Visible = true;
                this.data_trial++;
                btnStart.Text = "Start";
                TrialEnd();
            }
        }
        #endregion

        #region Radio button callbacks to configure the keyboard
        private void rbQwerty_CheckedChanged(object sender, EventArgs e)
        {
            if (rbQwerty.Checked)
            {
                this.resetKeyboard();
            }
        }
        private void rbAlpha_CheckedChanged(object sender, EventArgs e)
        {
            if (rbAlpha.Checked)
            {
                this.resetKeyboard();
            }
        }

        private void rbRandom_CheckedChanged(object sender, EventArgs e)
        {
            if (rbRandom.Checked)
            {
                this.resetKeyboard();
            }

        }
        private void rbSmall_CheckedChanged(object sender, EventArgs e)
        {
            if (rbSmall.Checked)
            {
                this.resetKeyboard();
            }
        }

        private void rbLarge_CheckedChanged(object sender, EventArgs e)
        {
            if (rbLarge.Checked)
            {
                this.resetKeyboard();
            }
        }
        #endregion

        #region Radio button callbacks to configure keyboard vs. mouse typing
        private void rbKeyboard_CheckedChanged(object sender, EventArgs e)
        {
            if (rbKeyboard.Checked)
            {
                data_category = "Keyboard typing";
            }
        }

        private void rbMouse_CheckedChanged(object sender, EventArgs e)
        {
            if (rbMouse.Checked)
            {
                data_category = "Mouse typing";
            }
        }
        #endregion



    }
}
